const isolateBackend = require('./isolate')
const fs = require('fs')
const { spawn } = require('child_process');
const registry_url = "10.129.6.5:5000/"

function runIsolate(filename) {
    return new Promise((resolve, reject) => {
        let timeStart = Date.now()
        let { isolate, context } = isolateBackend.createIsolate();

        fs.readFile(filename, 'utf-8', (err, data) => {

            if (err)
                reject(err);

            context.evalClosure(data).then(result => {
                
                let timeDifference = Math.ceil((Date.now() - timeStart))
                console.log("isolate time taken: ", timeDifference);
                resolve(result.result);
                
            }).catch(err => { reject(err) })
        });
    });

}

function runProcess(filename) {
    
    return new Promise((resolve, reject) => {
        let timeStart = Date.now()
        const process = spawn('node', [filename]);
        let result = "";
        process.stdout.on('data', (data) => {
            console.log(`stdout: ${data}`);
            result += data;
            let timeDifference = Math.ceil((Date.now() - timeStart))
            console.log("process time taken: ", timeDifference);
        });

        process.stderr.on('data', (data) => {
            console.error(`stderr: ${data}`);
            reject(data);
        });

        process.on('close', (code) => {
            console.log(`child process exited with code ${code}`);
            resolve(result);
        });
    })

}


function runContainer(imageName) {
    console.log(imageName);

    return new Promise((resolve, reject) => {
        let timeStart = Date.now()

        const process_checkImage = spawn('docker', ["inspect", registry_url + imageName])

        process_checkImage.on('close', (code) => {
            if (code != 0) {
                const process_pullImage = spawn('docker', ["pull", registry_url + imageName]);

                process_pullImage.stderr.on('data', (data) => {
                    console.error(`stderr: ${data}`);
                    reject(data);
                });

                process_pullImage.on('close', (code) => {
                    if (code != 0)
                        reject("error")
                    else {
                        const process = spawn('docker', ["run", "--name", imageName, registry_url + imageName]);
                        let result = "";
                        timeStart = Date.now()
                        process.stdout.on('data', (data) => {
                            console.log(`stdout: ${data}`);
                            let timeDifference = Math.ceil((Date.now() - timeStart))
                            console.log("container run time taken: ", timeDifference);
                            result += data;
                        });

                        process.stderr.on('data', (data) => {
                            console.error(`stderr: ${data}`);
                            reject(data);
                        });

                        process.on('close', (code) => {
                            resolve(result);
                        })
                    }
                    
                })
            } else {
                const process_checkContainer = spawn('docker', ['container', 'inspect', imageName]);
                process_checkContainer.on('close', (code) => {
                    if (code != 0) {
                        const process = spawn('docker', ["run", "--name", imageName, registry_url + imageName]);
                        let result = "";
                        timeStart = Date.now()
                        process.stdout.on('data', (data) => {
                            console.log(`stdout: ${data}`);
                            let timeDifference = Math.ceil((Date.now() - timeStart))
                            console.log("container run time taken: ", timeDifference);
                            result += data;
                        });

                        process.stderr.on('data', (data) => {
                            console.error(`stderr: ${data}`);
                            reject(data);
                        });

                        process.on('close', (code) => {
                            resolve(result);
                        })
                    } else {
                        const process = spawn('docker', ["start", "-a", imageName]);
                        let result = "";
                        timeStart = Date.now()
                        process.stdout.on('data', (data) => {
                            console.log(`stdout: ${data}`);
                            let timeDifference = Math.ceil((Date.now() - timeStart))
                            console.log("container run time taken: ", timeDifference);
                            result += data;
                        });

                        process.stderr.on('data', (data) => {
                            console.error(`stderr: ${data}`);
                            reject(data);
                        });

                        process.on('close', (code) => {
                            resolve(result);
                        })
                    }
                })
                
            }
            
        })

        
    })

}

module.exports.runContainer = runContainer;
module.exports.runProcess = runProcess;
module.exports.runIsolate = runIsolate;