'use strict';
const constants = require(".././constants.json")
const config = require('./config.json')
const libSupport = require('./lib')
libSupport.updateConfig()
const node_id = config.id
const execute = require('./execute')
const fs = require('fs')
const kafka = require('kafka-node')
const local_repository = __dirname + "/local_repository/"
const host_url = "http://" + constants.master_address + ":" + constants.master_port

let Producer = kafka.Producer,
    KeyedMessage = kafka.KeyedMessage,
    client = new kafka.KafkaClient({ 
        kafkaHost: constants.kafka_host,
        autoConnect: true
    }),
    producer = new Producer(client),
    Consumer = kafka.Consumer

libSupport.makeTopic(node_id).then(() => {
    console.log("node topic created")
    let consumer = new Consumer(client,
        [
            { topic: node_id, partition: 0, offset: 0 }
        ],
        [
            { autoCommit: true }
        ])
    consumer.on('message', function (message) {
        console.log(message);
        let topic = message.topic
        message = message.value
        message = JSON.parse(message)
        if (topic !== 'heartbeat') {
            let runtime = message.runtime
            let functionHash = message.functionHash
            let function_id = message.function_id

            if (message.type === "execute") {
                console.log("function_id", function_id);
                libSupport.download(host_url + "/repository/" + functionHash, local_repository + functionHash).then(() => {
                    startWorker(local_repository, functionHash, function_id, producer, runtime)
                })

            }
        }

    })
})
    

function startWorker(local_repository, functionHash,function_id, producer, runtime) {
    if (runtime === "isolate")
        execute.runIsolate(local_repository + functionHash).then(result => {
            producer.send([{
                topic: "response",
                messages: JSON.stringify({
                    status: "success",
                    result,
                    function_id
                })
            }], () => { })
        })
    else if (runtime === "process")
        execute.runProcess(local_repository + functionHash).then(result => {
            producer.send(
                [{
                    topic: "response",
                    messages: JSON.stringify({
                        status: "success",
                        result,
                        function_id
                    })
                }], () => { })
        })
    else if (runtime === "container")
        execute.runContainer(functionHash).then(result => {
            producer.send(
                [{
                    topic: "response",
                    messages: JSON.stringify({
                        status: "success",
                        result,
                        function_id
                    })
                }], () => { })
        })
    else {
        producer.send(
            [{
                topic: "response",
                messages: JSON.stringify({ status: "unknown runtime" })
            }], () => { })

        return
    }
}

function heartbeat() {
    let payload = [{
        topic: "heartbeat",
        messages: JSON.stringify({"address": node_id})
    }]
    producer.send(payload, function() {

    })
}

setInterval(heartbeat, 1000);