"use strict";

const express = require('express')
const bodyParser = require('body-parser')
const fileUpload = require('express-fileupload');
const constants = require('.././constants.json')
const fs = require('fs')
const { spawn } = require('child_process');
const morgan = require('morgan')
const mqtt = require('mqtt')
// const client = mqtt.connect('mqtt://' + constants.mqtt_url)
const app = express()
const libSupport = require('./lib')

let kafka = require('kafka-node'),
    Producer = kafka.Producer,
    KeyedMessage = kafka.KeyedMessage,
    client = new kafka.KafkaClient({ 
        kafkaHost: '10.129.6.5:9092',
        autoConnect: true
    }),
    producer = new Producer(client),
    Consumer = kafka.Consumer,
    consumer = new Consumer(client,
        [
            { topic: 'response', partition: 0, offset: 0}, { topic: 'heartbeat' }
        ],
        [
            { autoCommit: true }
        ])

let db = new Map()
app.use(morgan('combined'))
app.use(bodyParser.urlencoded({ extended: true }))
app.use(bodyParser.json())
const file_path = __dirname + "/repository/"

app.use('/repository', express.static(file_path));
app.use(fileUpload())

let requestQueue = []
let workerNodes = []

const WINDOW_SIZE = 10
const port = 8080
const registry_url = constants.registry_url

app.post('/serverless/deploy', (req, res) => {
    
    let runtime = req.body.runtime
    let file = req.files.serverless

    let functionHash = file.md5
    
    file.mv(file_path + functionHash, function (err) {
        if (err) {
            console.log(err);
            res.send("error").status(400)
        }
        else {
            if (runtime === "container") {
                deployContainer(file_path, functionHash)
                    .then(() => {
                        res.json({
                            status: "success",
                            function_id: functionHash
                        })
                    })
                    .catch(err => {
                        res.json({
                            status: "error",
                            reason: err}).status(400)
                    })
            } else {
                res.json({
                    status: "success",
                    function_id: functionHash
                })
            }
        }
    })
    
})

function deployContainer(path, imageName) {
    return new Promise((resolve, reject) => {
        let buildStart = Date.now()
        
        fs.writeFile('./repository/Dockerfile',
            `FROM node:latest
            WORKDIR /app
            COPY package.json /app
            RUN npm install
            COPY . /app
            
            CMD node ${imageName}`
            , function (err) {
                if (err) {
                    console.log("failed", err);
                    
                    reject(err);
                }
                else {
                    console.log('Dockerfile created');
                    const process = spawn('docker', ["build", "-t", registry_url + imageName, path]);

                    process.stdout.on('data', (data) => {
                        console.log(`stdout: ${data}`);

                    });

                    process.stderr.on('data', (data) => {
                        console.error(`stderr: ${data}`);
                    });

                    process.on('close', (code) => {
                        console.log(`child process exited with code ${code}`);
                        let timeDifference = Math.ceil((Date.now() - buildStart))
                        console.log("image build time taken: ", timeDifference);
                        const process_push = spawn('docker', ["push", registry_url + imageName]);

                        process_push.stdout.on('data', (data) => {
                            console.log(`stdout: ${data}`);

                        });

                        process_push.stderr.on('data', (data) => {
                            console.error(`stderr: ${data}`);
                        });

                        process_push.on('close', (code) => {
                            console.log("image pushed to repository");
                            resolve();
                        })
                        
                    });
                }
            });
    })
}

app.post('/serverless/execute/:id', (req, res) => {

    requestQueue.push({
        req, res
    })

    if (requestQueue.length >= WINDOW_SIZE)
        dispatch()
})


function dispatch() {
    let lookbackWindow = Math.min(WINDOW_SIZE, requestQueue.length)
    for (let i = 0; i < lookbackWindow; i++) {
        let {req, res} = requestQueue.shift()
	console.log(req.body)
        let runtime = req.body.runtime
        let functionHash = req.params.id
        let function_id = libSupport.makeid(20)
        console.log("Dispatching function with Id", function_id, runtime);
        let node_id = getAddress()
        let payload = [{
            topic: node_id,
            messages: JSON.stringify({
                "type": "execute",
                function_id,
                runtime, functionHash
            }),
            partition: 0
        }]
        producer.send(payload, () => {})
        db.set(function_id, res)
        
    }
}

function getAddress() {
    return workerNodes[Math.floor(Math.random() * workerNodes.length)];
}


// client.on('message', function (topic, message) {

//     if (topic === "response") {
//         message = JSON.parse(message)
//         console.log(message);
//         let res = db.get(message.function_id)
//         res.json({
//             "status": "success",
//             "reply": message.result
//         })
//         db.delete(message.function_id)
//     } else if (topic === "heartbeat") {
//         message = JSON.parse(message)
//         if (workerNodes.indexOf(message.address) === -1)  {
//             workerNodes.push(message.address)
//             console.log(workerNodes);
// 	}
//     }
// })

app.listen(port, () => console.log(`Server listening on port ${port}!`))

// client.on('connect', function () {
//     client.subscribe("response")
//     client.subscribe("heartbeat")
// })

consumer.on('message', function (message) {
    // console.log(message);
    let topic = message.topic
        message = message.value
    if (topic === "response") {
        message = JSON.parse(message)
        console.log(message);
        let res = db.get(message.function_id)
        res.json({
            "status": "success",
            "reply": message.result
        })
        db.delete(message.function_id)
    } else if (topic === "heartbeat") {
        message = JSON.parse(message)
        
        if (workerNodes.indexOf(message.address) === -1)  {
            workerNodes.push(message.address)
            console.log(workerNodes);
        }
    }
});

setInterval(dispatch, 2000);
